@@ -71,6 +71,7 @@ class ModelSet(object):
         self._default_unit = dict()
         self._default_unit["ratio"] = u.dimensionless_unscaled
         self._default_unit["intensity"] = _OBS_UNIT_
+        self._user_added_models = dict()
 
     @property
     def description(self):
@@ -83,16 +84,16 @@ class ModelSet(object):
     @property
     def code(self):
         """The PDR code that created this ModelSet, e.g. 'KOSMA-tau'
-        
+
         :rtype: str
         """
         return self._tabrow["PDR code"]
-    
+
     @property
     def name(self):
         """The name of this ModelSet
 
-        :rtype: str 
+        :rtype: str
         """
         return self._tabrow["name"]
 
@@ -103,16 +104,16 @@ class ModelSet(object):
         :rtype: str
         """
         return self._tabrow["version"]
-    
+
     @property
     def medium(self):
         """The medium type of this model, e.g. 'constant density', 'clumpy', 'non-clumpy'
-        
+
         :rtype: str
         """
         return self._tabrow["medium"]
-    
-    @property 
+
+    @property
     def z(self):
         """The metallicity of this ModelSet
 
@@ -132,7 +133,7 @@ class ModelSet(object):
     def table(self):
         """The table containing details of the models in this ModelSet.
 
-        :rtype: :class:`astropy.table.Table` 
+        :rtype: :class:`astropy.table.Table`
         """
         return self._table
 
@@ -140,16 +141,16 @@ class ModelSet(object):
     def identifiers(self):
         """Table of lines and continuum that are included in ratio models of this ModelSet. Only lines and continuum that are part of ratios are included in this list.   For a separate list of line and continuum intensity models see :meth:`~pdrtpy.modelset.ModelSet.supported_intensities`.
 
-        :rtype: :class:`astropy.table.Table` 
+        :rtype: :class:`astropy.table.Table`
         """
         return self._identifiers
 
     @property
     def supported_intensities(self):
-        """Table of lines and continuum that are covered by this ModelSet and have models separate from 
+        """Table of lines and continuum that are covered by this ModelSet and have models separate from
         the any ratio model they might be in.
 
-        :rtype: :class:`astropy.table.Table` 
+        :rtype: :class:`astropy.table.Table`
         """
         return self._supported_lines
 
@@ -157,10 +158,18 @@ class ModelSet(object):
     def supported_ratios(self):
         """The emission ratios that are covered by this ModelSet
 
-        :rtype: :class:`astropy.table.Table` 
+        :rtype: :class:`astropy.table.Table`
         """
         return self._supported_ratios
 
+    @property
+    def user_added_models(self):
+        """Show which models have been added to this ModelSet by the user
+
+        :rtype: list
+        """
+        return list(self._user_added_models.keys())
+
     def ratiocount(self,m):
         """The number of valid ratios in this ModelSet, given a list of observation (:class:`~pdrtpy.measurement.Measurement`) identifiers.
 
@@ -181,14 +190,14 @@ class ModelSet(object):
         """
         if not isinstance(m, collections.abc.Iterable) or isinstance(m, (str, bytes)) :
             raise Exception("m must be an array of strings")
-            
+
         for q in itertools.product(m,m):
             if q[0] == "FIR" and (q[1] == "OI_145" or q[1] == "OI_63") and "CII_158" in m:
                 s = q[1] + "+CII_158/" + q[0]
             else:
                 s = q[0]+"/"+q[1]
             if s in self.table["ratio"]:
-                yield s 
+                yield s
 
     def find_files(self,m,ext="fits"):
         """Find the valid model ratios files in this ModelSet for a given list of measurement IDs.  See :meth:`~pdrtpy.measurement.Measurement.id`
@@ -214,8 +223,8 @@ class ModelSet(object):
             if s in self.table["ratio"]:
                 fullpath = self._tabrow["path"]+self.table.loc[s]["filename"]+"."+ext
                 tup = (s,fullpath)
-                yield tup 
-            
+                yield tup
+
     def model_ratios(self,m):
         '''Return the model ratios that match the input Measurement ID list.  You must provide at least 2 Measurements IDs
 
@@ -232,7 +241,7 @@ class ModelSet(object):
         return ratios
 
     def model_intensities(self,m):
-        '''Return the model intensities in this ModelSet that match the input Measurement ID list.  
+        '''Return the model intensities in this ModelSet that match the input Measurement ID list.
         This method will return the intersection of the input list and the list of supported lines.
 
         :param m: list of string :class:`~pdrtpy.measurement.Measurement` IDs, e.g., ["CII_158","OI_145","CS_21"]
@@ -241,26 +250,28 @@ class ModelSet(object):
         :rtype: list
         '''
         # get intersection of input list and supported lines
-        return list(set(m) & set(self._supported_lines["intensity label"])) 
+        return list(set(m) & set(self._supported_lines["intensity label"]))
 
     def get_model(self,identifier,unit=None,ext="fits"):
         '''Get a specific model by its identifier
 
-        :param identifier: a :class:`~pdrtpy.measurement.Measurement` ID. It can be an intensity or a ratio,
-   #,habing_unit,draine_unit,mathis_unit      e.g., "CII_158","CI_609/FIR"
+        :param identifier: a :class:`~pdrtpy.measurement.Measurement` ID. It can be an intensity or a ratio, e.g., "CII_158","CI_609/FIR".
         :type identifier: str
         :returns: The model matching the identifier
         :rtype: :class:`~pdrtpy.measurement.Measurement`
         :raises: KeyError if identifier not found in this ModelSet
         '''
 
+        if identifier in self._user_added_models:
+            return self._user_added_models[identifier]
+
         if identifier not in self.table["ratio"]:
             raise KeyError(f"{identifier} is not in this ModelSet")
 
         d = model_dir()
         _thefile = d+self._tabrow["path"]+self.table.loc[identifier]["filename"]+"."+ext
         _title = self._table.loc[identifier]['title']
-        if unit is None:
+        if unit is None or unit == "":
             # make a guess at the unit
             if '/' in identifier:
                 unit = self._default_unit["ratio"]
@@ -274,6 +285,8 @@ class ModelSet(object):
             else:
                 modeltype = "intensity"
         _model = Measurement.read(_thefile,title=_title,unit=unit,identifier=identifier)
+        #if _model.unit=="":
+        #    _model.unit = u.Unit("adu")#self._default_unit["ratio"]
         _wcs = _model.wcs
         _model.header["MODELTYP"] = modeltype
         _model.modeltype = modeltype
@@ -297,11 +310,11 @@ class ModelSet(object):
                 _model.header["CUNIT1"] = "cm^-3"
                 _wcs.wcs.cunit[0] = u.Unit("cm^-3")
             else:
-                _model.header["CUNIT1"] = str(_wcs.wcs.cunit[0])  
+                _model.header["CUNIT1"] = str(_wcs.wcs.cunit[0])
             if _wcs.wcs.cunit[1] == "":
                 _model.header["CUNIT2"] = "Draine"
             else:
-                _model.header["CUNIT2"] = str(_wcs.wcs.cunit[1])  
+                _model.header["CUNIT2"] = str(_wcs.wcs.cunit[1])
         else:
             # copy wcs cunit to header. used later.
             _model.header["CUNIT1"] = str(_wcs.wcs.cunit[0])
@@ -314,9 +327,9 @@ class ModelSet(object):
 
         :param identifiers: list of string :class:`~pdrtpy.measurement.Measurement` IDs, e.g., ["CII_158","OI_145","CS_21"]
         :type identifiers: list
-        :param model_type: indicates which type of model is requested one of 'ratio' or 'intensity' 
+        :param model_type: indicates which type of model is requested one of 'ratio' or 'intensity'
         :type model_type: str
-        :returns: The matching models as a list of :class:`~pdrtpy.measurement.Measurement`. 
+        :returns: The matching models as a list of :class:`~pdrtpy.measurement.Measurement`.
         :rtype: list
         :raises: KeyError if identifiers not found in this ModelSet
         '''
@@ -333,7 +346,7 @@ class ModelSet(object):
             a.extend(self.model_intensities(identifiers))
         if model_type == "ratio" or model_type == "both":
             a.extend(self.model_ratios(identifiers))
-            
+
         if model_type == "intensity" or model_type == "ratio":
             _unit = self._default_unit[model_type]
         else:
@@ -343,6 +356,47 @@ class ModelSet(object):
 
         return models
 
+    def add_model(self,identifier,model,title,overwrite=False):
+        r"""Add your own model to this ModelSet.
+
+        :param identifier: a :class:`~pdrtpy.measurement.Measurement` ID. It can be an intensity or a ratio, e.g., "CII_158","CI_609/FIR".
+        :type identifier: str
+        :param model:  the model to add.  If a string, this must be the fully-qualified path of a FITS file.  If a :class:`~pdrtpy.measurement.Measurement` it must have the same CTYPEs and CUNITs as the models in the ModelSet(?).
+        :type model: str or :class:`~pdrtpy.measurement.Measurement`
+        :param title: A formatted string (e.g., LaTeX) describing this observation that can be used for plotting. Python r-strings are accepted, e.g., r'$^{13}$CO(3-2)'  would give :math:`^{13}{\rm CO(3-2)}`.
+    :type title: str
+        :param overwrite:  Whether to overwrite the model if the identifier already exists in the ModelSet or has been previously added.  Default: False
+        :type overwrite: bool
+        """
+        if identifier not in self.table["ratio"] and identifier not in self._user_added_models:
+            self._really_add_model(identifier,model,title)
+        elif identifier in self._user_added_models and not overwrite:
+            raise Exception(f"{identifier} was previously added to this ModelSet. If you wish to overwrite it, use overwrite=True")
+        elif identifier in self.table["ratio"] and not overwrite:
+            raise Exception(f"{identifier} is already in the {self.name} ModelSet. If you wish to overwrite it, use overwrite=True")
+        else:
+            #print(f"Overwriting {identifier}.")
+            self._really_add_model(identifier,model,title)
+
+
+    def _really_add_model(self,identifier,model,title):
+        print("Adding user model %s"%identifier)
+        if type(model) is str:
+            m = Measurement.read(model,identifier=identifier)
+        else:
+            m = model
+        self._user_added_models[identifier] = m
+        if "/" in identifier: # it's a ratio
+            if identifier in self._supported_ratios["ratio label"]:
+                # ack, there should be a way just to replace title but I can't get Table.loc to work.
+                index = np.where(self._supported_ratios["ratio label"] == identifier)[0][0]
+                self._supported_ratios.remove_row(index)
+            self._supported_ratios.add_row([title,identifier])
+        else:
+            if identifier in self._supported_lines["intensity label"]:
+                index = np.where(self._supported_lines["intensity label"] == identifier)[0][0]
+                self._supported_lines.remove_row(index)
+            self._supported_lines.add_row([title,identifier])
 
     def _find_ratio_elements(self,m):
         # TODO handle case of OI+CII/FIR so it is not special cased in lineratiofit.py
@@ -354,16 +408,16 @@ class ModelSet(object):
         """
         if not isinstance(m, collections.abc.Iterable) or isinstance(m, (str, bytes)) :
             raise Exception("m must be an array of strings")
-            
+
         for q in itertools.product(m,m):
             s = q[0]+"/"+q[1]
             z = dict()
             if s in self.table["ratio"]:
                 z={"numerator":self.table.loc[s]["numerator"],
                    "denominator":self.table.loc[s]["denominator"]}
-                yield z 
+                yield z
 
-    def _get_ratio_elements(self,m):   
+    def _get_ratio_elements(self,m):
         """Get the valid model numerator,denominator pairs in this ModelSet for a given list of measurement IDs. See :meth:`~pdrtpy.measurement.Measurement.id`
 
         :param m: list of string :class:`~pdrtpy.measurement.Measurement` IDs, e.g. ["CII_158","OI_145","FIR"]
@@ -372,7 +426,7 @@ class ModelSet(object):
         """
         if not isinstance(m, collections.abc.Iterable) or isinstance(m, (str, bytes)) :
             raise Exception("m must be an array of strings")
-        k = list()   
+        k = list()
         for q in itertools.product(m,m):
             s = q[0]+"/"+q[1]
             if s in self.table["ratio"]:
@@ -383,12 +437,12 @@ class ModelSet(object):
         return k
 
     def _get_oi_cii_fir(self,m,k):
-        """Utility method for determining ratio elements, to handle special 
+        """Utility method for determining ratio elements, to handle special
            case of ([O I] 63 micron + [C II] 158 micron)/I_FIR.
 
         :param m: list of string :class:`~pdrtpy.measurement.Measurement` IDs, e.g. ["CII_158","OI_145","FIR"]
         :type m: list
-        :param k: list of existing ratios to append to  
+        :param k: list of existing ratios to append to
         :type k: list
         """
         if "CII_158" in m and "FIR" in m:
@@ -437,7 +491,7 @@ class ModelSet(object):
         for i in range(len(t1['title'])):
             if '/' in t1['title'][i]:
                 t1['title'][i] = t1['title'][i][0:t1['title'][i].index('/')]
-        
+
         t2 = Table([self._table['title'][matching_rows],d],copy=True)
         # remove numerator from title (everything before and including /)
         for i in range(len(t2['title'])):
@@ -452,13 +506,13 @@ class ModelSet(object):
 
     @property
     def is_wk2006(self):
-        """method to indicate this is a wk2006 model, to deal with quirks 
+        """method to indicate this is a wk2006 model, to deal with quirks
            of that modelset
-        
+
            :returns: True if it is.
         """
         return self.name == "wk2006"
-        
+
 
     # ============= Static Methods =============
     @staticmethod
@@ -469,8 +523,8 @@ class ModelSet(object):
     @staticmethod
     def all_sets():
         """Return a table of the names and descriptions of available ModelSets (not just this one)
-        
-        :rtype:`:class:~astropy.table.Table`
+
+        :rtype: :class:`~astropy.table.Table`
         """
         t = get_table("all_models.tab")
         t.remove_column("path")
