@@ -1,4 +1,4 @@
-#todo: 
+#todo:
 # keywords for show_both need to be arrays. ugh.
 #
 # allow levels to be percent?
@@ -14,18 +14,15 @@ import numpy as np
 import numpy.ma as ma
 import scipy.stats as stats
 
-import matplotlib.figure
-import matplotlib.colors as mpcolors
+#import matplotlib.figure
+#import matplotlib.colors as mpcolors
 import matplotlib.cm as mcm
-from matplotlib import ticker
+#from matplotlib import ticker
 from matplotlib.lines import Line2D
 
-from astropy.nddata.utils import Cutout2D
-from astropy.io import fits
 import astropy.wcs as wcs
 import astropy.units as u
 from astropy.units import UnitsWarning
-from astropy.nddata import NDDataArray, CCDData, NDUncertainty, StdDevUncertainty, VarianceUncertainty, InverseVariance
 
 from .plotbase import PlotBase
 from .modelplot import ModelPlot
@@ -33,12 +30,13 @@ from .. import pdrutils as utils
 
 
 class LineRatioPlot(PlotBase):
-    """Class to plot various results from PDR Toolbox model fitting.
-    
+    """LineRatioPlot plots the results of :class:`~pdrtpy.tools.lineratiofit.LineRatioFit`.  It can plot maps of fit results, observations with errors on top of models, chi-square and confidence intervals and more.
+
 
     :Keyword Arguments:
+
     The methods of this class can take a variety of optional keywords.  See the general `Plot Keywords`_ documentation
-       
+
     """
 
     def __init__(self,tool):
@@ -65,7 +63,7 @@ class LineRatioPlot(PlotBase):
         ms = self._tool.modelset
         if id not in ms.supported_lines["intensity label"]:
             raise KeyError(f"{id} is not in the ModelSet of your LineRatioFit")
-        
+
         model = ms.get_models([id],model_type="intensity")
         kwargs_opts = {'title': self._tool._modelset.table.loc[id]["title"], 'colorbar':True}
         kwargs_opts.update(kwargs)
@@ -75,14 +73,14 @@ class LineRatioPlot(PlotBase):
 
     def modelratio(self,id,**kwargs):
         """Plot one of the model ratios
-     
+
            :param id: the ratio identifier, such as ``CII_158/CO_32``.
            :type id: str
            :param \**kwargs: see class documentation above
            :raises KeyError: if is id not in existing model ratios
 
         """
-        if len(self._tool._modelratios[id].shape) == 0:
+        if self._tool._modelratios[id].shape == (1,):  #does this ever occur??
             return self._tool._modelratios[id]
 
         kwargs_opts = {'title': self._tool._modelset.table.loc[id]["title"], 'units': u.dimensionless_unscaled , 'colorbar':True}
@@ -98,7 +96,7 @@ class LineRatioPlot(PlotBase):
            :type id: - str
            :raises KeyError: if id is not in existing observed ratios
         """
-        if len(self._tool._observedratios[id].shape) == 0:
+        if self._tool._observedratios[id].shape == (1,0) or self._tool.has_vectors:
             return self._tool._observedratios[id]
 
         kwargs_opts = {'title': self._tool._modelset.table.loc[id]["title"], 'units': u.dimensionless_unscaled , 'colorbar':False}
@@ -121,7 +119,7 @@ class LineRatioPlot(PlotBase):
         kwargs_opts.update(kwargs)
 
         # handle single pixel or multi-pixel non-map cases.
-        if len( self._tool._density.shape) == 0 or self._tool.has_vectors:
+        if self._tool._density.shape == (1,) or self._tool.has_vectors:
             return utils.to(kwargs_opts['units'],self._tool._density)
 
         tunit=u.Unit(kwargs_opts['units'])
@@ -144,11 +142,11 @@ class LineRatioPlot(PlotBase):
         kwargs_opts.update(kwargs)
 
         # handle single pixel or multi-pixel non-map cases.
-        if len( self._tool.radiation_field.shape) == 0 or self._tool.has_vectors:
+        if self._tool.radiation_field.shape == (1,) or self._tool.has_vectors:
             return utils.to(kwargs_opts['units'],self._tool.radiation_field)
 
         if kwargs_opts['title'] is None:
-            rad_title = utils.get_rad(kwargs_opts['units']) 
+            rad_title = utils.get_rad(kwargs_opts['units'])
             tunit=u.Unit(kwargs_opts['units'])
             kwargs_opts['title'] = r"{0} [{1:latex_inline}]".format(rad_title,tunit)
 
@@ -158,17 +156,17 @@ class LineRatioPlot(PlotBase):
     #    """Make a line plot of chisq as a function of G0 or n for a given pixel"""
     #    axes = {"G0":0,"n":1}
     #    axis = axes[xaxis] #yep key error if you do it wrong
-    #        
+    #
 
     #@TODO refactor this method with reduced_chisq()
-    def chisq(self, **kwargs):           
+    def chisq(self, **kwargs):
         '''Plot the :math:`\chi^2` map that was computed by the
         :class:`~pdrtpy.tool.lineratiofit.LineRatioFit` tool.
-        
+
         '''
 
         kwargs_opts = {'units': None,
-                       'aspect': 'equal',
+                       'aspect': 'auto',
                        'image':True,
                        'contours': True,
                        'label': False,
@@ -196,24 +194,28 @@ class LineRatioPlot(PlotBase):
             data = self._tool.chisq(min=False)
             self._modelplot._plot_no_wcs(data,header=None,**kwargs_opts)
             # Put a crosshair where the chisq minimum is.
-            # To do this we first get the array index of the minimum
-            # then use WCS to translate to world coordinates.
-            [row,col] = np.where(self._tool._chisq==self._tool._chisq_min.value)
-            mywcs = wcs.WCS(data.header)
-            # Suppress WCS warning about 1/cm3 not being FITS
-            warnings.simplefilter('ignore',category=UnitsWarning)
-            logn,logrf = mywcs.array_index_to_world(row,col)
-            warnings.resetwarnings()
-            n = (10**logn.value[0])*u.Unit(logn.unit.to_string())
-            rf = (10**logrf.value[0])*u.Unit(logrf.unit.to_string())
+            if False:
+                # To do this we first get the array index of the minimum
+                # then use WCS to translate to world coordinates.
+                [row,col] = np.where(self._tool._chisq==self._tool._chisq_min.value)
+                mywcs = wcs.WCS(data.header)
+                # Suppress WCS warning about 1/cm3 not being FITS
+                warnings.simplefilter('ignore',category=UnitsWarning)
+                logn,logrf = mywcs.array_index_to_world(row,col)
+                warnings.resetwarnings()
+                n = (10**logn.value[0])*u.Unit(logn.unit.to_string())
+                rf = (10**logrf.value[0])*u.Unit(logrf.unit.to_string())
+            # since the minimum can now be between pixels, we use the value
+            # of radiation field and density directly.  (Why didn't we do this before?)
             if kwargs_opts['xaxis_unit'] is not None:
-                x = n.to(kwargs_opts['xaxis_unit']).value
+                x = utils.to(self._tool._density,kwargs_opts['xaxis_unit']).value
             else:
-                x = n.value
+                x = self._tool._density.value
             if kwargs_opts['yaxis_unit'] is not None:
-                y = rf.to(kwargs_opts['yaxis_unit']).value
+                y = utils.to(kwargs_opts['yaxis_unit'],self._tool._radiation_field).value
             else:
-                y = rf.value
+                y = self._tool._radiation_field.value
+
             #print("Plot x,y %s,%s"%(x,y))
 
             if kwargs_opts['title'] is None:
@@ -229,11 +231,11 @@ class LineRatioPlot(PlotBase):
     def reduced_chisq(self, **kwargs):
         '''Plot the reduced :math:`\chi^2` map that was computed by the
         :class:`~pdrtpy.tool.lineratiofit.LineRatioFit` tool.
-        
+
         '''
 
         kwargs_opts = {'units': None,
-                       'aspect': 'equal',
+                       'aspect': 'auto',
                        'image':True,
                        'contours': True,
                        'label': False,
@@ -249,7 +251,7 @@ class LineRatioPlot(PlotBase):
         kwargs_opts.update(kwargs)
         if self._tool.has_vectors:
             raise NotImplementedError("Plotting of chi-square is not yet implemented for vector Measurements.")
-            
+
         if self._tool.has_maps:
             if 'title' not in kwargs:
                 kwargs_opts['title'] = r'$\chi_\nu^2$ (dof=%d)'%self._tool._dof
@@ -262,40 +264,43 @@ class LineRatioPlot(PlotBase):
 
             self._modelplot._plot_no_wcs(data,header=None,**kwargs_opts)
             # Put a crosshair where the chisq minimum is.
-            # To do this we first get the array index of the minimum
-            # then use WCS to translate to world coordinates.
-            [row,col] = np.where(self._tool._reduced_chisq==self._tool._reduced_chisq_min.value)
-            mywcs = wcs.WCS(data.header)
-            # Suppress WCS warning about 1/cm3 not being FITS
-            warnings.simplefilter('ignore',category=UnitsWarning)
-            logn,logrf = mywcs.array_index_to_world(row,col)
-            warnings.resetwarnings()
-            # logn, logrf are Quantities of the log(density) and log(radiation field),
-            # respectively.  The model default units are cm^-2 and erg/s/cm^-2. 
-            # These must be converted to plot units based on user input 
-            # xaxis_unit and yaxis_unit. 
-            # Note: multiplying by n.unit causes the ValueError:
-            # "The unit '1/cm3' is unrecognized, so all arithmetic operations with it are invalid."
-            # Yet by all other measures this appears to be a valid unit. 
-            # The workaround is to us to_string() method.
-            n = (10**logn.value[0])*u.Unit(logn.unit.to_string())
-            rf = (10**logrf.value[0])*u.Unit(logrf.unit.to_string())
+            if False:
+                # To do this we first get the array index of the minimum
+                # then use WCS to translate to world coordinates.
+                [row,col] = np.where(self._tool._reduced_chisq==self._tool._reduced_chisq_min.value)
+                mywcs = wcs.WCS(data.header)
+                # Suppress WCS warning about 1/cm3 not being FITS
+                warnings.simplefilter('ignore',category=UnitsWarning)
+                logn,logrf = mywcs.array_index_to_world(row,col)
+                warnings.resetwarnings()
+                # logn, logrf are Quantities of the log(density) and log(radiation field),
+                # respectively.  The model default units are cm^-2 and erg/s/cm^-2.
+                # These must be converted to plot units based on user input
+                # xaxis_unit and yaxis_unit.
+                # Note: multiplying by n.unit causes the ValueError:
+                # "The unit '1/cm3' is unrecognized, so all arithmetic operations with it are invalid."
+                # Yet by all other measures this appears to be a valid unit.
+                # The workaround is to us to_string() method.
+                n = (10**logn.value[0])*u.Unit(logn.unit.to_string())
+                rf = (10**logrf.value[0])*u.Unit(logrf.unit.to_string())
+            # since the minimum can now be between pixels, we use the value
+            # of radiation field and density directly.  (Why didn't we do this before?)
             if kwargs_opts['xaxis_unit'] is not None:
-                x = n.to(kwargs_opts['xaxis_unit']).value
+                x = utils.to(self._tool._density,kwargs_opts['xaxis_unit']).value
             else:
-                x = n.value
+                x = self._tool._density.value
             if kwargs_opts['yaxis_unit'] is not None:
-                y = rf.to(kwargs_opts['yaxis_unit']).value
+                y = utils.to(kwargs_opts['yaxis_unit'],self._tool._radiation_field).value
             else:
-                y = rf.value
-
+                y = self._tool._radiation_field.value
+            #print("Plot x,y %s,%s"%(x,y))
             if kwargs_opts['title'] is None:
                 kwargs_opts['title'] = r'$\chi_\nu^2$ (dof=%d)'%self._tool._dof
             label = r'$\chi_{\nu,min}^2$ = %.2g @ (n,FUV) = (%.2g,%.2g)'%(self._tool._reduced_chisq_min.value,x,y)
-            self._axis[0].scatter(x,y,c='r',marker='+',s=200,linewidth=2,label=label)
+            self._modelplot.axis[0].scatter(x,y,c='r',marker='+',s=200,linewidth=2,label=label)
             # handle legend locally
             if kwargs_opts['legend']:
-                legend = self._axis[0].legend(loc='upper center',title=kwargs_opts['title'])
+                legend = self._modelplot.axis[0].legend(loc='upper center',title=kwargs_opts['title'])
             self._figure = self._modelplot.figure
             self._axis = self._modelplot.axis
 
@@ -308,7 +313,7 @@ class LineRatioPlot(PlotBase):
         _reset = [True,False]
 
         kwargs_opts = {'image':True,
-                       'aspect': 'equal',
+                       'aspect': 'auto',
                        'contours': False,
                        'label': False,
                        'levels': None,
@@ -326,7 +331,7 @@ class LineRatioPlot(PlotBase):
 
         kwargs_opts['index'] = _index[1]
         kwargs_opts['reset'] = _reset[1]
- 
+
         d = self.density(units=units[1],**kwargs_opts)
         # @todo don't return for plots.  print for non-plots
         return (rf,d)
@@ -334,7 +339,7 @@ class LineRatioPlot(PlotBase):
     def confidence_intervals(self,**kwargs):
         '''Plot the confidence intervals from the :math:`\chi^2` map computed by the
         :class:`~pdrtpy.tool.lineratiofit.LineRatioFit` tool. Default levels:  [50., 68., 80., 95., 99.]
-        
+
         **Currently only works for single-pixel Measurements**
         '''
 
@@ -342,7 +347,7 @@ class LineRatioPlot(PlotBase):
             raise NotImplementedError("Plotting of confidence intervals is not yet implemented for maps or vectors.")
 
         kwargs_opts = {'units': None,
-                       'aspect': 'equal',
+                       'aspect': 'auto',
                        'image':False,
                        'contours': True,
                        'label': True,
@@ -354,7 +359,7 @@ class LineRatioPlot(PlotBase):
                        'xaxis_unit': None,
                        'yaxis_unit': None,
                        'title':  "Confidence Intervals"}
-        
+
         # ensure levels are in ascending order
         kwargs_opts['levels'] = sorted(kwargs_opts['levels'])
         kwargs_opts.update(kwargs)
@@ -365,12 +370,13 @@ class LineRatioPlot(PlotBase):
         self._modelplot._plot_no_wcs(data=confidence,header=None,**kwargs_opts)
         self._figure = self._modelplot.figure
         self._axis = self._modelplot.axis
-    
-    def overlay_all_ratios(self,**kwargs):
-        '''Overlay all the measured ratios and their errors on the :math:`(n,G_0)` space. 
 
-        **Currently only works for single-pixel Measurements**
+    def overlay_all_ratios(self,**kwargs):
+        '''Overlay all the measured ratios and their errors on the :math:`(n,F_{FUV})` space.
+
+        This only works for single-valued Measurements; an overlay for multi-pixel doesn't make sense.
         '''
+#NB: could have position and area though.
 
         if self._tool.has_maps or self._tool.has_vectors:
             raise NotImplementedError("Plotting of ratio overlays is not yet implemented for maps or vectors.")
@@ -386,24 +392,66 @@ class LineRatioPlot(PlotBase):
                        'norm': None,
                        'title': None,
                        'reset': True,
-                       'legend': True}
+                       'legend': True,
+                        'bbox_to_anchor':None,
+                        'loc': 'upper center'}
 
         kwargs_opts.update(kwargs)
         # force this as ncols !=1 makes no sense.
         kwargs_opts['ncols'] = 1
 
-        i =0 
-        for key,val in self._tool._modelratios.items():
-            kwargs_opts['measurements'] = [self._tool._observedratios[key]]
-            if i > 0: kwargs_opts['reset']=False
-            # pass the index of the contour color to use via the "secret" colorcounter keyword.
-            self._modelplot._plot_no_wcs(val,header=None,colorcounter=i,**kwargs_opts)
-            i = i+1
+        i = 0
+        _measurements = list()
+        _models = list()
+        # get_model will correctly raise exception if m.id not in ModelSet
+        meas_passed = False
+        if kwargs_opts.get('measurements',None) is not None:
+            # avoid modifying a passed parameter
+            _measurements = deepcopy(kwargs_opts['measurements'])
+            meas_passed = True
+            for m in _measurements:
+                if i > 0: 
+                    kwargs_opts['reset']=False
+                val = self._tool.modelset.get_model(m.id)
+                _models.append(val)
+                kwargs_opts['measurements'] = [utils.convert_if_necessary(m)]
+                self._modelplot._plot_no_wcs(_models[i],header=None,colorcounter=i,**kwargs_opts)
+                i = i+1
+            km = kwargs_opts.pop('measurements')
+        #for m in _measurements:
+        #    print("A ",type(m))
 
+        for key,val in self._tool._modelratios.items():
+            #kwargs_opts['measurements'] = [self._tool._observedratios[key]]
+            #print(" K ",type(self._tool._observedratios[key]))
+            #_measurements.append(self._tool._observedratios[key])
+            if i > 0: 
+                kwargs_opts['reset']=False
+            # pass the index of the contour color to use via the "secret" colorcounter keyword.
+            self._modelplot._plot_no_wcs(val,header=None,colorcounter=i,**kwargs_opts,measurements=[self._tool._observedratios[key]])
+            i = i+1
+        #for m in _measurements:
+        #    print("B ",type(m))
         if kwargs_opts['legend']:
             lines = [Line2D([0], [0], color=c, linewidth=3, linestyle='-') for c in kwargs_opts['meas_color'][0:i]]
-            labels = [self._tool._modelratios[k].title for k in self._tool._modelratios]
-            self._plt.legend(lines, labels,loc='upper center',title='Observed Ratios')
+            labels = list()
+            if meas_passed:
+                # THIS WILL EXCEPT IF m.id NOT IN MODELSET WHICH IS VERY POSSIBLE!
+                for m in _measurements:
+                    try:
+                        tt = self._tool.modelset.get_model(m.id).title
+                    except Exception:
+                        tt = m.id
+                    labels.append(tt)
+                #labels = [m.id for m in _measurements]
+                title = "Observed Ratios and Intensities"
+            else:
+                title = "Observed Ratios"
+            labels.extend([self._tool._modelratios[k].title for k in self._tool._modelratios])
+            #print("LABELS ",labels)
+            self._plt.legend(lines, labels,loc=kwargs_opts['loc'],
+                             bbox_to_anchor=kwargs_opts['bbox_to_anchor'],
+                             title=title)
         self._figure = self._modelplot.figure
         self._axis = self._modelplot.axis
 
@@ -443,7 +491,8 @@ class LineRatioPlot(PlotBase):
 
         for key,val in self._tool._modelratios.items():
             axidx = kwargs_opts['index']-1
-            if kwargs_opts['index'] > 1: kwargs_opts['reset'] = False
+            if kwargs_opts['index'] > 1: 
+                kwargs_opts['reset'] = False
             m = self._tool._model_files_used[key]
             kwargs_opts['measurements'] = [self._tool._observedratios[key]]
             self._modelplot._plot_no_wcs(val,header=None,**kwargs_opts)
@@ -475,7 +524,7 @@ class LineRatioPlot(PlotBase):
         '''generic plotting method used by other plot methods'''
 
         kwargs_plot = {'show' : 'data' # or 'mask' or 'error'
-                      } 
+                      }
 
         kwargs_opts = {'units' : None,
                        'image':True,
@@ -485,7 +534,7 @@ class LineRatioPlot(PlotBase):
                        'title': None
                        }
 
-        kwargs_contour = {'levels': None, 
+        kwargs_contour = {'levels': None,
                           'colors': ['white'],
                           'linewidths': 1.0}
 
@@ -526,14 +575,14 @@ class LineRatioPlot(PlotBase):
         min_ = np.nanmin(km)
         max_ = np.nanmax(km)
 
-        kwargs_imshow = { 'origin': 'lower', 
+        kwargs_imshow = { 'origin': 'lower',
                           'norm': 'simple',
                           'stretch': 'linear',
-                          'vmin': min_, 
+                          'vmin': min_,
                           'vmax': max_,
                           'cmap': 'plasma',
                           'aspect': 'auto'}
- 
+
         kwargs_subplot = {'nrows': 1,
                           'ncols': 1,
                           'index': 1,
@@ -543,7 +592,7 @@ class LineRatioPlot(PlotBase):
 
         # delay merge until min_ and max_ are known
         kwargs_imshow.update(kwargs)
-        kwargs_imshow['norm']=self._get_norm(kwargs_imshow['norm'],km, 
+        kwargs_imshow['norm']=self._get_norm(kwargs_imshow['norm'],km,
                                              kwargs_imshow['vmin'],kwargs_imshow['vmax'],
                                              kwargs_imshow['stretch'])
 
@@ -572,12 +621,12 @@ class LineRatioPlot(PlotBase):
             # suppress errors and warnings about unused keywords
             #@todo need a better solution for this, it is not scalable.
             #push onto a stack?
-            for kx in ['units', 'image', 'contours', 'label', 'title','linewidths','levels','nrows','ncols', 
+            for kx in ['units', 'image', 'contours', 'label', 'title','linewidths','levels','nrows','ncols',
                        'index', 'reset','colors','colorbar','show','yaxis_unit','xaxis_unit',
                        'constrained_layout','figsize','stretch','legend']:
                 kwargs_imshow.pop(kx,None)
             # eliminate deprecation warning.  vmin,vmax are passed to Normalization object.
-            if kwargs['norm'] is not None:
+            if kwargs_opts['norm'] is not None:
                 kwargs_imshow.pop('vmin',None)
                 kwargs_imshow.pop('vmax',None)
             im=self._axis[axidx].imshow(km,**kwargs_imshow)
@@ -586,7 +635,7 @@ class LineRatioPlot(PlotBase):
 
         if kwargs_opts['contours']:
             if kwargs_contour['levels'] is None:
-                # Figure out some autolevels 
+                # Figure out some autolevels
                 kwargs_contour['levels'] = self._autolevels(km,'log')
 
             # suppress errors and warnings about unused keywords
@@ -599,7 +648,7 @@ class LineRatioPlot(PlotBase):
             if kwargs_opts['label']:
                 self._axis[axidx].clabel(contourset,contourset.levels,inline=True,fmt='%1.1e')
 
-        if kwargs_opts['title'] is not None: 
+        if kwargs_opts['title'] is not None:
             #self.figure.subplots_adjust(top=0.95)
             #self._axis[axidx].set_title(kwargs_opts['title'])
             # Using ax.set_title causes the title to be cut off.  No amount of
@@ -610,5 +659,5 @@ class LineRatioPlot(PlotBase):
         if k.wcs is not None:
             self._axis[axidx].set_xlabel(k.wcs.wcs.lngtyp)
             self._axis[axidx].set_ylabel(k.wcs.wcs.lattyp)
-     
+
         #self._figure.tight_layout()
