@@ -15,10 +15,11 @@ from .. import pdrutils as utils
 
 
 class ModelPlot(PlotBase):
-    """ ModelPlot is a tool for exploring sets of models.  It can plot individual intensity or ratio models, phase-space diagrams, and optionally overlay observations.   Units are seamlessly transformed, so you can plot in Habing units, Draine units, or any conformable quantity.  ModelPlot does not require model fitting with :class:`~pdrtpy.tool.lineratiofit.LineRatioFit` first.
+    """ModelPlot is a tool for exploring sets of models.  It can plot individual intensity or ratio models, phase-space diagrams, and optionally overlay observations.   Units are seamlessly transformed, so you can plot in Habing units, Draine units, or any conformable quantity.  ModelPlot does not require model fitting with :class:`~pdrtpy.tool.lineratiofit.LineRatioFit` first.
 
     :Keyword Arguments:
-    The methods of this class can take a variety of optional keywords.  See the general `Plot Keywords`_ documentation
+
+    The methods of this class can take a variety of optional keywords.  See the general `Plot Keywords`_ documentation.
     """
     def __init__(self,modelset,figure=None,axis=None):
         """Init method
@@ -31,7 +32,7 @@ class ModelPlot(PlotBase):
         self._figure = figure
         self._axis = axis
         #print(utils.habing_unit)
-    
+
     def plot(self,identifier,**kwargs):
         """Plot a model intensity or ratio
 
@@ -57,8 +58,8 @@ class ModelPlot(PlotBase):
         """
         ms = self._modelset
         model = ms.get_model(identifier)
-        kwargs_opts = {'title': ms.table.loc[identifier]["title"], 
-                       'colorbar':True, 
+        kwargs_opts = {'title': ms.table.loc[identifier]["title"],
+                       'colorbar':True,
                        'contours':True,
                        'colors':['white'],
                        'meas_color': [self._CB_color_cycle[0]],
@@ -78,7 +79,7 @@ class ModelPlot(PlotBase):
             labels = list()
             if kwargs_opts['contours']:
                 lines.append(Line2D([0], [0], color=kwargs_opts['colors'][0], linewidth=3, linestyle='-'))
-                labels.append("model")
+                labels.append(f"{self._modelset.name} model")
             if kwargs_opts['measurements'] is not None:
                 lines.append(Line2D([0], [0], color=kwargs_opts['meas_color'][0], linewidth=3, linestyle='-'))
                 labels.append("observed")
@@ -102,13 +103,13 @@ class ModelPlot(PlotBase):
             if type(meas[0]) is not Measurement:
                 raise TypeError("measurement keyword value must be a list of Measurements")
             if (model[identifier]._unit != meas[0].unit ):
-                raise TypeError(f"Model and Measurement for {identifier} have different units: ({model._unit},{meas_unit})")
+                raise TypeError(f"Model and Measurement for {identifier} have different units: ({model[identifier]._unit},{meas[0].unit})")
             if identifier != meas[0].id:
                 msg = f"Identifiers of model {identifier} and supplied Measurement {meas[0].id} do not match. Plotting anyway."
                 warnings.warn(msg)
 
-        kwargs_opts = {'title': ms.table.loc[identifier]["title"], 
-                       'colorbar':True, 
+        kwargs_opts = {'title': ms.table.loc[identifier]["title"],
+                       'colorbar':True,
                        'contours':True,
                        'colors':['white'],
                        'meas_color': [self._CB_color_cycle[0]],
@@ -134,13 +135,13 @@ class ModelPlot(PlotBase):
 
 
     def overlay(self,measurements,**kwargs):
-        '''Overlay one or more single-pixel measurements in the model space ($n,G_0). 
+        '''Overlay one or more single-pixel measurements in the model space :math:`(n,F_{FUV})`.
 
-        :param measurements: a list of one or more :class:`pdrtpy.measurement.Measurement` to overlay.
+        :param measurements: a list of one or more :class:`~pdrtpy.measurement.Measurement` to overlay.
         :type measurements: list
-        :param shading: Controls how measurements and errors are drawn.  If shading is zero, Measurements will be drawn in solid contour for the value and dashed for the +/- errors. If shading is between 0 and 1, Measurements are drawn with as filled contours representing the size of the errors (see :meth:`~matplotlib.pyplot.contourf`) with alpha set to the shading value.  Default value: 0.4
+        :param shading: Controls how measurements and errors are drawn.  If ``shading`` is zero, Measurements will be drawn in solid contour for the value and dashed for the +/- errors. If ``shading`` is between 0 and 1, Measurements are drawn with as filled contours representing the size of the errors (see :meth:`matplotlib.pyplot.contourf`) with alpha set to the ``shading`` value.  Default value: 0.4
         :type shading: float
-        
+
         '''
 
         kwargs_opts = {'units': None,
@@ -167,19 +168,22 @@ class ModelPlot(PlotBase):
         if utils._has_H2(ids) and self._modelset.is_wk2006:
             warnings.warn("Trimming all model grids to match H2 grid: log(n) = 1-5, log(G0) = 1-5")
             utils._trim_all_to_H2(models)
-        i =0 
+        i =0
         nratio = 0
         nintensity = 0
         for val in models:
             if np.size(meas[val.id].data) != 1:
                 raise ValueError(f"Can't plot {val.id}. This method only works with single pixel Measurements [len(measurement.data) must be 1]")
-            if i > 0: kwargs_opts['reset']=False
+            if i > 0: 
+                kwargs_opts['reset']=False
             # pass the index of the contour color to use via the "secret" colorcounter keyword.
             self._plot_no_wcs(val,header=None,
                               measurements=[utils.convert_if_necessary(meas[val.id])],
                               colorcounter=i,**kwargs_opts)
-            if val.modeltype == "ratio": nratio=nratio+1
-            if val.modeltype == "intensity": nintensity=nintensity+1
+            if val.modeltype == "ratio": 
+                nratio=nratio+1
+            if val.modeltype == "intensity": 
+                nintensity=nintensity+1
             i = i+1
         if kwargs_opts['legend']:
             if nratio == 0 and nintensity >0:
@@ -193,8 +197,8 @@ class ModelPlot(PlotBase):
             self._plt.legend(lines, labels,loc='upper center',title='Observed '+word)
 
 
-    # note when plotting the units as axis labels, the order is not what we specify in _OBS_UNIT because astropy's Unit class 
-    # sorts by power .  They have a good reason for this (hashing), but it does mean we get sub-optimal unit ordering.  
+    # note when plotting the units as axis labels, the order is not what we specify in _OBS_UNIT because astropy's Unit class
+    # sorts by power .  They have a good reason for this (hashing), but it does mean we get sub-optimal unit ordering.
     # There is a possible workaround, but it must be custom for each CompositeUnit.https://github.com/astropy/astropy/issues/1578
     def phasespace(self,identifiers,
                  nax1_clip=[10,1E7]*u.Unit("cm-3"),
@@ -206,7 +210,7 @@ class ModelPlot(PlotBase):
         :type identifiers: list of str
         :param nax1_clip: The range of model densities on NAXIS1 to show in the plot. For most model NAXIS1 is hydrogen number density $n_H$ in cm$^{-3}$.  For ionized gas models, it is electron temperature $T_e$ in K.  Must be given as a range of astropy quanitities.  Default: [10,1E7]*Unit("cm-3")
         :type nax1_clip: array-like, must contain Quantity
-        :param nax2_clip: The range of model parameters on NAXIS2 to show in the plot.  For most models NAXIS2 is radiation field intensities in Habing or cgs units.  For ionized gas models, it is electron volume density $n_e$.  Must be given as a range of astropy quantities.  Default: nax1_clip=[10,1E6]*utils.habing_unit. 
+        :param nax2_clip: The range of model parameters on NAXIS2 to show in the plot.  For most models NAXIS2 is radiation field intensities in Habing or cgs units.  For ionized gas models, it is electron volume density $n_e$.  Must be given as a range of astropy quantities.  Default: nax1_clip=[10,1E6]*utils.habing_unit.
         :type nax2_clip:  array-like, must contain Quantity
         :param reciprocal: Whether or not the plot the reciprocal of the model on each axis.  Given as a pair of booleans.  e.g. [False,True] means don't flip the quantity X axis, but flip quantity the Y axis.  i.e. if the model is "CII/OI", and reciprocal=True then the axis will be "OI/CII".  Default: [False, False]
         :type reciprocal: array-like bool
@@ -214,7 +218,7 @@ class ModelPlot(PlotBase):
         The following keywords are supported as \*\*kwargs:
 
         :param measurements: A list of two :class:`~pdrtpy.measurement.Measurement`, one for each `identifier`, that will be used to plot a data point on the grid. At least two Measurements, one for x and one for y, must be given.  Subsequent Measurements must also be paired since they represent x and y, e.g `[m1x, m1y, m2x, m2y,...]`. Measurement *data* and *uncertainty* members may be arrays.  Default: None
-        :type measurements: array-like of :class:`~pdrtpy.measurment.Measurement`.  
+        :type measurements: array-like of :class:`~pdrtpy.measurment.Measurement`.
         :param errorbar: Plot error bars when given measurements. Default: True
         :type errorbar: bool
         :param fmt: The format to use when plotting Measurement data. There should be one for each pair of Measurements. See :meth:`matplotlib.axes.Axes.plot` for examples. Default is 'sk' for all points.
@@ -246,9 +250,10 @@ class ModelPlot(PlotBase):
                        'figsize':(8,5),
                        'linewidth': 2.0,
                        'capsize': 5.0,
-                       'markersize': 8.0
+                       'markersize': 8.0,
+                       'aspect': 'auto'
                        }
-    
+
         kwargs_opts.update(kwargs)
         # various input checks
         if len(list(identifiers)) != 2:
@@ -266,15 +271,17 @@ class ModelPlot(PlotBase):
                     raise TypeError(f"Model and Measurement for {m.id} have different units: ({mdict[m.id]._unit},{m.unit})")
 
         # Now we need to find the model points that fall within the user-specified NAXIS limits.
-        # First get the x,y of the models 
+        # First get the x,y of the models
         xlog,ylog=self._get_xy_from_wcs(models[0],quantity=True,linear=False)
         xlin,ylin=self._get_xy_from_wcs(models[0],quantity=True,linear=True)
-        
+
         x_is_log = False
         y_is_log = False
-        if 'log' in models[0].wcs.wcs.ctype[0]: x_is_log = True
-        if 'log' in models[0].wcs.wcs.ctype[1]: y_is_log = True
-        
+        if 'log' in models[0].wcs.wcs.ctype[0]: 
+            x_is_log = True
+        if 'log' in models[0].wcs.wcs.ctype[1]: 
+            y_is_log = True
+
         # linear and log units are same so doesn't matter which is used for conversion
         dcc=nax1_clip.to(xlog.unit)
         rcc=nax2_clip.to(ylog.unit)
@@ -295,7 +302,7 @@ class ModelPlot(PlotBase):
         # Intersection of these two arrays contain the indices of desired model plot points.
         xi2=np.intersect1d(xi,x2)
         yi2=np.intersect1d(yi,y2)
-        
+
         self._figure,self._axis = self._plt.subplots(nrows=1,ncols=1,figsize=kwargs_opts['figsize'])
         linesN=[]
         linesG=[]
@@ -341,7 +348,7 @@ class ModelPlot(PlotBase):
             else:
                 yy=models[1][j,xi2[0]:xi2[-1]+1]
             linesG.extend(self._axis.loglog(xx,yy,label=label,lw=2,ls='--'))
-            
+
         # plot the input measurement with error bar. Keywords are
         # [m1x,m1y,m2x,m2y,...]
         # [fmt1,fmt2,...]
@@ -380,8 +387,8 @@ class ModelPlot(PlotBase):
                     _y = kwargs_opts['measurements'][k]
                 # collect the args
                 args.extend([_x,_y,fmt[i]])
-                # Plot the error bars.  Since, unlike axis.loglog(), axis.errorbar() can't 
-                # take a *args, we have to call this each time. 
+                # Plot the error bars.  Since, unlike axis.loglog(), axis.errorbar() can't
+                # take a *args, we have to call this each time.
                 # Note use of zorder to ensure points are on top of lines.
                 if kwargs_opts['errorbar']:
                     self._axis.errorbar(x=_x.data,y=_y.data,xerr=_x.error,yerr=_y.error,
@@ -390,6 +397,7 @@ class ModelPlot(PlotBase):
                 i=i+1
             # the data points
             dataline = self._axis.loglog(*args,zorder=5,markersize=kwargs_opts['markersize'])
+            self._axis.set_aspect(kwargs_opts['aspect'])
             self._axis.set_xscale('log')
             self._axis.set_yscale('log')
             self._axis.tick_params(axis='both',direction='in',which='both')
@@ -399,17 +407,17 @@ class ModelPlot(PlotBase):
                             color='k',alpha=0.33)
             self._axis.grid(b=True,which='minor',axis='both',lw=kwargs_opts['linewidth']/2,
                             color='k',alpha=0.22,linestyle='--')
-            
-            
-        if kwargs_opts['legend']:            
+
+
+        if kwargs_opts['legend']:
             # Manually build the legend. Create the column headers for the legend
             # and blank handles and labels to take up space for the headers and
             # when the number of density traces and radiation field traces
-            # are not equal. 
+            # are not equal.
             title1 = models[0].wcs.wcs.ctype[0]
             if "_" in title1:
                 title1 = r"${\rm "+title1+"}$"
-            unit1="["+nax1_clip.unit.to_string("latex_inline")+"]" 
+            unit1="["+nax1_clip.unit.to_string("latex_inline")+"]"
             rs = nax2_clip.unit.to_string()
             rsl = nax2_clip.unit.to_string("latex_inline")
             if "G0" in  models[0].wcs.wcs.ctype[1] or "FUV" in models[0].wcs.wcs.ctype[1]:
@@ -419,7 +427,7 @@ class ModelPlot(PlotBase):
                 if "_" in title2:
                     title2 = r"${\rm "+title2+"}$"
             unit2="["+rsl+"]"
-        
+
             handles,labels=self._axis.get_legend_handles_labels()
             phantom = [self._axis.plot([],marker="", markersize=0,ls="",lw=0)[0]]*2
             lN = len(linesN)
@@ -440,7 +448,7 @@ class ModelPlot(PlotBase):
                 labels.insert(lN,title2)
                 labels = [title1,unit1]+labels + blank
                 linesN = phantom + linesN
-                linesG = phantom + linesG + phantom2  
+                linesG = phantom + linesG + phantom2
             elif diff < 0: # more radiation fields than densities
                 labels = labels[0:lN]+blank+labels[lN:]
                 labels.insert(lN+adiff,unit2)
@@ -462,14 +470,14 @@ class ModelPlot(PlotBase):
             for vpack in leg._legend_handle_box.get_children():
                 for hpack in vpack.get_children()[:2]:
                     hpack.get_children()[0].set_width(0)
-        # Put the plot title on if given.            
+        # Put the plot title on if given.
         if kwargs_opts['title'] is not None:
             self._axis.set_title(kwargs_opts['title'])
 
-        
+
     def _get_xy_from_wcs(self,data,quantity=False,linear=False):
         """Get the x,y axis vectors from the WCS of the input data.
-       
+
         :param data: the input image
         :type data: :class:`astropy.io.fits.ImageHDU`, :class:`astropy.nddata.CCDData`, or :class:`~pdrtpy.measurement.Measurement`.
         :param quantity: If True, return the arrays as :class:`astropy.units.Quantity`. If False, the returned arrays are :class:`numpy.ndarray`.
@@ -480,8 +488,8 @@ class ModelPlot(PlotBase):
         :rtype: :class:`numpy.ndarray` or :class:`astropy.units.Quantity`
         """
         return utils.get_xy_from_wcs(data,quantity,linear)
-    
-        
+
+
     #@todo allow data to be an array? see overlay()
     def _plot_no_wcs(self,data,header=None,**kwargs):
         '''generic plotting method for images with no WCS, used by other plot methods'''
@@ -490,7 +498,7 @@ class ModelPlot(PlotBase):
         if _dataheader is None  and header is None:
             raise Exception("Either your data must have a header dictionary or you must provide one via the header parameter")
         # input header supercedes data header, under assumption user knows what they are doing.
-        if header is not None: 
+        if header is not None:
             _header = deepcopy(header)
             if getattr(data,"wcs",None) is None:
                 data.wcs = wcs.WCS(_dataheader)
@@ -520,10 +528,11 @@ class ModelPlot(PlotBase):
                        'ylim':None,
                        'legend': False,
                        'meas_color': ['#4daf4a'],
-                       'shading': 0.4
+                       'shading': 0.4,
+                       'test':False
                        }
 
-        kwargs_contour = {'levels': None, 
+        kwargs_contour = {'levels': None,
                           'colors': ['white'],
                           'linewidths': 1.0}
 
@@ -561,19 +570,20 @@ class ModelPlot(PlotBase):
         min_ = np.nanmin(km)
         max_ = np.nanmax(km)
 
-        kwargs_imshow = { 'origin': 'lower', 
+        kwargs_imshow = { 'origin': 'lower',
                           'norm': 'simple',
                           'stretch': 'linear',
-                          'vmin': min_, 
+                          'vmin': min_,
                           'vmax': max_,
                           'cmap': 'plasma',
-                          'aspect': 'equal'}
- 
+                          'aspect': 'auto'}
+
         kwargs_subplot = {'nrows': 1,
                           'ncols': 1,
                           'index': 1,
                           'reset': True,
-                          'constrained_layout': True
+                          'constrained_layout': False,
+                          'projection': data.wcs
                          }
 
         # delay merge until min_ and max_ are known
@@ -589,30 +599,31 @@ class ModelPlot(PlotBase):
         axidx = kwargs_subplot['index']-1
         if kwargs_subplot['reset']:
 # @todo can probably consolidate this
-            self._figure,self._axis = self._plt.subplots(kwargs_subplot['nrows'],kwargs_subplot['ncols'],
-                                        figsize=kwargs_subplot['figsize'],
-                                        subplot_kw={'aspect':kwargs_imshow['aspect']},
-                                        constrained_layout=kwargs_subplot['constrained_layout'])
+            self._figure,self._axis = self._plt.subplots(
+                kwargs_subplot['nrows'],kwargs_subplot['ncols'],
+                figsize=kwargs_subplot['figsize'],
+                subplot_kw={'aspect':kwargs_imshow['aspect']},
+                constrained_layout=kwargs_subplot['constrained_layout'])
 
 
         # Make sure self._axis is an array because we will index it below.
         if type(self._axis) is not np.ndarray:
             self._axis = np.array([self._axis])
 
-        # When using ncols>1, either the index needs to be 2-d 
+        # When using ncols>1, either the index needs to be 2-d
         # or the axis array needs to be 1-d.  This takes the second approach:
         if len(self._axis.shape) > 1:
             self._axis = self._axis.flatten()
 
         ax1='1'
         ax2='2'
-            
+
         # make the x and y axes.  Since the models are computed on a log grid, we
         # use logarithmic ticks.
         x,y = self._get_xy_from_wcs(data,quantity=True,linear=True)
         locmaj = ticker.LogLocator(base=10.0, subs=(1.0, ),numticks=10)
-        locmin = ticker.LogLocator(base=10.0, subs=np.arange(2, 10)*.1,numticks=10) 
-        
+        locmin = ticker.LogLocator(base=10.0, subs=np.arange(2, 10)*.1,numticks=10)
+
         #allow unit conversion of density axis
         xax_unit = u.Unit(_header['CUNIT'+ax1])
         # cover the base where we had to erase the wcs unit to avoid FITS error
@@ -626,12 +637,12 @@ class ModelPlot(PlotBase):
             xax_unit = u.Unit(kwargs_opts['xaxis_unit'])
 
             # Convert the unit-aware grid to the desired units and set X to the value (so it's no longer a Quantity)
-            #x = temp_x.to(xax_unit).value  
+            #x = temp_x.to(xax_unit).value
             x = x.to(xax_unit)
 
         # Set the x label appropriately, use LaTeX inline formatting
         xlab = r"{0} [{1:latex_inline}]".format(_header['CTYPE'+ax1],xax_unit)
-        
+
         yax_unit = u.Unit(_header['CUNIT'+ax2])
         if y._unit is None or y._unit is u.dimensionless_unscaled:
             y._unit = yax_unit
@@ -648,7 +659,7 @@ class ModelPlot(PlotBase):
             yax_unit = u.Unit(yunit)
 
             # Convert the unit-aware grid to the desired units and set Y to the value (so it's no longer a Quantity)
-            #y = temp_y.to(yax_unit).value  
+            #y = temp_y.to(yax_unit).value
             y = y.to(yunit)
 
         # Set the y label appropriately, use LaTeX inline formatting
@@ -675,8 +686,10 @@ class ModelPlot(PlotBase):
             im = self._axis[axidx].pcolormesh(x.value,y.value,km,cmap=kwargs_imshow['cmap'],
                                               norm=_norm,shading='auto')
             if kwargs_opts['colorbar']:
-                #cbar = self._wcs_colorbar(im,self._axis[axidx],pad=0.05,width="5%") looks like crap
-                cbar = self._figure.colorbar(im,ax=self._axis[axidx])#,format=ticker.ScalarFormatter(useMathText=True))
+                if kwargs_opts['test']:
+                    cbar = self._wcs_colorbar(im,self._axis[axidx],pad=0.1,width="5%") #looks like crap
+                else:
+                    cbar = self._figure.colorbar(im,ax=self._axis[axidx])#,format=ticker.ScalarFormatter(useMathText=True))
 
                 if kwargs_imshow['norm'].lower() != "log":
                     #avoid AttributeError: 'LogFormatterSciNotation' object has no attribute 'set_powerlimits'
@@ -686,15 +699,15 @@ class ModelPlot(PlotBase):
                     cbar.update_ticks()
                 if "BUNIT" in _header:
                     lstr = u.Unit(_header["BUNIT"]).to_string('latex_inline')
-                    
+
                     cbar.ax.set_ylabel(lstr,rotation=90)
 
         if kwargs_opts['contours']:
             if kwargs_contour['levels'] is None:
-                # Figure out some autolevels 
+                # Figure out some autolevels
                 kwargs_contour['levels'] = self._autolevels(km,'log')
 
-            # suppress warnings about unused keywords and potential error 
+            # suppress warnings about unused keywords and potential error
             # about cmap not being None. Also norm being a string will cause an error
             # in matplotlib==3.3.1+
             #@todo need a better solution for this, it is not scalable.
@@ -724,7 +737,7 @@ class ModelPlot(PlotBase):
         if measurements is not None:
             lstyles = ['--','-','--']
             # for serial calls to plot_no_wcs in an outside method (i.e. lineratioplot.overlay_all_ratios),
-            # we need to keep track of the index for the measurement color, otherwise we always 
+            # we need to keep track of the index for the measurement color, otherwise we always
             # select color index 0, resulting in all measurements contours having the same color.
             # this is a kluge but it's all I can think of right now.
             if 'colorcounter' in kwargs:
@@ -741,7 +754,7 @@ class ModelPlot(PlotBase):
                 # See https://github.com/mpound/pdrtpy/issues/23
                     cset2 = self._axis[axidx].contour(x.value,y.value,k.data,levels=[m.levels[1]], colors=colors, alpha=kwargs_opts['shading'])
                 else:
-                    cset = self._axis[axidx].contour(x.value,y.value,k.data,levels=m.levels, 
+                    cset = self._axis[axidx].contour(x.value,y.value,k.data,levels=m.levels,
                                                      linestyles=lstyles, colors=colors)
                 jj=jj+1
 
